#include <string.h>
#include <stdio.h>
#include <stdlib.h>
#include "rssmy.h"
#include <visa.h>

/*= Rohde & Schwarz Signal Generator SMY ====================================*/
/* LabWindows/CVI 3.1 Instrument Driver                                      */
/* Original Release: March 1996                                              */
/* By: Pavel Kedron, Technical University Ostrava, Czech Republic,           */
/*              e-mail address: pavel.kedron@vsb.cz                          */
/*              Ph (+420) 69-699-4882                                        */
/*                                                                           */
/* This driver is based on the SMX driver and has been modified to allow     */
/* the multi-instance use of the driver. Modified for SMY specific           */
/* requirements. Changed control ranges and added new phase modulation       */
/* support functions.                                                        */
/*                                                                           */
/* For LabWindows Support in the United States call:                         */
/*              National Instruments, Austin, Texas                          */
/*              Ph (800) 433-3488    Fax (512) 794-5678                      */
/*                                                                           */
/* Modification History:                                                     */
/* Modifications:        Modified for MDLL Compatibility                     */
/*                                                                           */
/* Modification Date:    June 21, 1996                                       */
/*                                                                           */
/* Modified by:          DS, National Instruments, Austin, Texas             */
/*                       Phone (800) 433-3488  Fax (512) 794-5678            */
/*								             */
/*---------------------------------------------------------------------------*/
/* Modifications:        Update to the latest VXIpnp standard.               */
/*                                                                           */
/* Modification Date:    July 08 , 1999                                      */
/*                                                                           */
/* Modified by:          Zdenek Rykala, ELCOM, a.s., Czech Republic          */
/*                       Phone +420 69 6996146  Fax +420 69 6996147          */
/*                                                                           */                                                                          
/*****************************************************************************/
/*---------------------------------------------------------------------------*/
/* Modifications:      	 Modified for SMY01 and SMY02                        */
/*	                     Rev 2.00, 07/1999 									 */
/*                  	 Fixed bug in function rssmy_setRFLevel              */ 
/*                                                                           */   
/* Modification Date:    July 27 , 1999                                      */
/*                                                                           */
/* Modified by:          Werner Klotzsche, Rohde & Schwarz                   */
/*                       Phone + 49 89 4129 2418  Fax +49 89 4129 2131       */
/*                                                                           */                                                                          
/*****************************************************************************/
/*---------------------------------------------------------------------------*/
/* Modifications:      	 Modified for SMY01 and SMY02                        */
/*	                     Rev 2.01, 01/2002                                   */
/*                       Fixed range in function rssmy_setFMMod              */
/*                                                                           */
/* Modification Date:    January , 2001                                      */
/*                                                                           */
/* Modified by:          Werner Klotzsche, Rohde & Schwarz                   */
/*                       Phone + 49 89 4129 12418  Fax +49 89 4129 612418    */
/*                                                                           */                                                                          
/*****************************************************************************/
	/*- Driver specific macros ----------------------------------------------*/
#define CHECKERR(fCal) if (rssmy_status = (fCal), rssmy_status < 0) \
					   	   return rssmy_status; else

#define VICHECKERR(fCal) if (rssmy_status = (fCal), rssmy_status < 0) \
					   	   return rssmy_initCleanUp (rmSession, instrSession, rssmy_status); else

	/*- Driver specific constants -------------------------------------------*/
#define RSSMY_REVISION			"Rev 2.01, 01/2002, CVI 5.0.1" /* Instrument driver revision */
#define BUFFER_SIZE 			 128  						  /* File I/O buffer size */

	/*- 488.2 Event Status Register (ESR) Bits ------------------------------*/
#define IEEE_488_2_ERROR_BITS	0x3C

/*****************************************************************************/
/*= INSTRUMENT-DEPENDENT COMMAND ARRAYS =====================================*/
/*****************************************************************************/
static ViString levelUnits[] = {"DBM", "V"};
static ViString levelComm[] = {"L", "L:E"};
static ViString nonIntrComm[] = {"N", "F"};
static ViString refFreqComm[] = {"E", "I"};
static ViString AMTypeComm[] = {"OFF", "I", "E:A", "E:D", "D:A", "D:D"};
static ViString FMTypeComm[] = {"OFF", "I", "E:A", "E:D", "D:A", "D:D"};
static ViString PMTypeComm[] = {"OFF", "N;B:ON", "I;B:ON"};
static ViString phMTypeComm[] = {"OFF", "I", "E", "D"};
static ViString directionComm[] = {"DEC", "INC"};
static ViString parameterComm[] = {"AF", "RF", "L", "AM", "FM", "PHM"};
static ViString stateComm[] = {"OFF", "ON"};

/*****************************************************************************/
/*= INSTRUMENT-DEPENDENT STATUS/RANGE STRUCTURE  ============================*/
/*****************************************************************************/
/* rssmy_stringValPair is used in the ifr68xx_errorMessage function          */
/* rssmy_statusDataRanges is used to track session dependent status &ranges  */
/*===========================================================================*/
typedef struct  rssmy_stringValPair
{
   ViStatus stringVal;
   ViString stringName;
}  rssmy_tStringValPair;

struct rssmy_statusDataRanges {
    ViBoolean 	checkStatus;
    ViChar 		instrDriverRevision[256];
};

typedef struct rssmy_statusDataRanges *rssmy_instrRange;

/*****************************************************************************/
/*= UTILITY ROUTINE DECLARATIONS (Non-Exportable Functions) =================*/
/*****************************************************************************/
ViBoolean rssmy_invalidViBooleanRange (ViBoolean val);
ViBoolean rssmy_invalidViReal64Range (ViReal64 val, ViReal64 min, ViReal64 max);
ViBoolean rssmy_invalidViInt32Range (ViInt32 val, ViInt32 min, ViInt32 max);

/*****************************************************************************/
/*------------- INSTRUMENT-DEPENDENT UTILITY ROUTINE DECLARATIONS -----------*/
/*****************************************************************************/
ViStatus rssmy_initCleanUp (ViSession openRMSession, ViPSession openInstrSession, ViStatus currentStatus);
ViStatus rssmy_defaultInstrSetup (ViSession openInstrSession);
ViStatus rssmy_check_status (ViSession instrSession);

/*****************************************************************************/
/*====== USER-CALLABLE FUNCTIONS (Exportable Functions) =====================*/
/*****************************************************************************/
/*=========================================================================*/
/* Function: Initialize                                                    */
/* Purpose:  This function opens the instrument, queries the instrument    */
/*           for its ID, and initializes the instrument to a known state.  */
/*=========================================================================*/
ViStatus _VI_FUNC rssmy_init (ViRsrc resourceName, ViBoolean IDQuery,
                              ViBoolean resetDevice, ViPSession instrSession)
{
    ViStatus	rssmy_status = VI_SUCCESS;
    ViSession 	rmSession = 0;

    /*- Check input parameter ranges ----------------------------------------*/
    if (rssmy_invalidViBooleanRange (IDQuery))
        return VI_ERROR_PARAMETER2;
    if (rssmy_invalidViBooleanRange (resetDevice))
        return VI_ERROR_PARAMETER3;

	/*- Open instrument session ---------------------------------------------*/
    CHECKERR( viOpenDefaultRM (&rmSession));

    if ((rssmy_status = viOpen (rmSession, resourceName, VI_NULL, VI_NULL, instrSession)) < 0)
		{
        viClose (rmSession);
        return rssmy_status;
		}

    /*- Configure VISA Formatted I/O ----------------------------------------*/
    VICHECKERR( viSetAttribute (*instrSession, VI_ATTR_TMO_VALUE, 10000));
    VICHECKERR( viSetBuf (*instrSession, VI_READ_BUF|VI_WRITE_BUF, 4000));
    VICHECKERR( viSetAttribute (*instrSession, VI_ATTR_WR_BUF_OPER_MODE, VI_FLUSH_ON_ACCESS));
    VICHECKERR( viSetAttribute (*instrSession, VI_ATTR_RD_BUF_OPER_MODE, VI_FLUSH_ON_ACCESS));

	/*- Identification Query ------------------------------------------------*/
    if (IDQuery)
    	{
    	ViByte		buffer[BUFFER_SIZE];
    	ViUInt32	retCnt;
    	
        VICHECKERR( viWrite (*instrSession, (ViByte *)"*IDN?", 5, &retCnt));
        VICHECKERR( viRead (*instrSession, buffer, BUFFER_SIZE, &retCnt));

		if (!strstr((ViChar *)buffer, "ROHDE&SCHWARZ,SMY"))
            return rssmy_initCleanUp (rmSession, instrSession, VI_ERROR_FAIL_ID_QUERY);
    	}

	/*- Reset instrument ----------------------------------------------------*/
    if (resetDevice)
        VICHECKERR( rssmy_reset (*instrSession));
    else  /*- Send Default Instrument Setup ---------------------------------*/
        VICHECKERR( rssmy_defaultInstrSetup (*instrSession));

    return rssmy_status;
}

/*=========================================================================*/
/* Function: Output Waveform                                               */
/* Purpose:  This function is the high-level example of the use of this    */
/*           driver's functions. It allows you to configure and output the */
/*           waveform.                                                     */
/*=========================================================================*/
ViStatus _VI_FUNC rssmy_outputWaveform (ViSession instrSession,
                                        ViReal64 radioFreq, ViReal64 audioFreq,
                                        ViReal64 rfLevel, ViBoolean rfLevelUnits,
                                        ViBoolean rfLevelType, ViBoolean rfOutput,
                                        ViBoolean afOutput)
{
    ViStatus rssmy_status = VI_SUCCESS;
    ViReal64 min = (rfLevelUnits) ? 3.0e-8 : -140.0,
			 max = (rfLevelUnits) ? 4.0 : 25.0;

    /*- Check input parameter ranges ----------------------------------------*/
    if (rssmy_invalidViReal64Range (radioFreq, 5.0e3, 2.08e9))
        return VI_ERROR_PARAMETER2;
    if (rssmy_invalidViReal64Range (audioFreq, 0.1, 5.0e5))
        return VI_ERROR_PARAMETER3;
	if (rssmy_invalidViReal64Range (rfLevel, min, max))
        return VI_ERROR_PARAMETER4;
	if (rssmy_invalidViBooleanRange (rfLevelUnits))
        return VI_ERROR_PARAMETER5;
	if (rssmy_invalidViBooleanRange (rfLevelType))
        return VI_ERROR_PARAMETER6;
	if (rssmy_invalidViBooleanRange (rfOutput))
        return VI_ERROR_PARAMETER7;
	if (rssmy_invalidViBooleanRange (afOutput))
        return VI_ERROR_PARAMETER8;

    CHECKERR( rssmy_setRF (instrSession, radioFreq));
    CHECKERR( rssmy_setAF (instrSession, audioFreq));
    CHECKERR( rssmy_setRFLevel (instrSession, rfLevel, rfLevelUnits, rfLevelType));
    CHECKERR( rssmy_rfOut (instrSession, rfOutput));
    CHECKERR( rssmy_afOut (instrSession, afOutput));

    return rssmy_status;
}

/*=========================================================================*/
/* Function: RF Setting                                                    */
/* Purpose:  This function sets the frequrency of the RF output.           */
/*=========================================================================*/
ViStatus _VI_FUNC rssmy_setRF (ViSession instrSession, ViReal64 rfValue)
{
	ViStatus	rssmy_status = VI_SUCCESS;
	ViChar		buffer[BUFFER_SIZE];

	/*- Check input parameter ranges ----------------------------------------*/
    if (rssmy_invalidViReal64Range (rfValue, 5.0e3, 2.08e9))
        return VI_ERROR_PARAMETER2;

    sprintf (buffer, "RF %lfHZ", rfValue);
	CHECKERR( viWrite( instrSession, (ViByte *)buffer, strlen( buffer), VI_NULL));
	CHECKERR( rssmy_check_status( instrSession));

    return rssmy_status;
}

/*=========================================================================*/
/* Function: RF Level Setting                                              */
/* Purpose:  This function sets the level of the RF output.                */
/*=========================================================================*/
ViStatus _VI_FUNC rssmy_setRFLevel (ViSession instrSession, ViReal64 rfLevel,
                                    ViBoolean rfLevelUnits, ViBoolean rfLevelType)
{
	ViStatus rssmy_status = VI_SUCCESS;
	ViReal64 min = (rfLevelUnits) ? 3.0e-8 : -140.0,
			 max = (rfLevelUnits) ? 4 : 25.0;
	ViChar   buffer[BUFFER_SIZE];

	/*- Check input parameter ranges ----------------------------------------*/
    if (rssmy_invalidViReal64Range (rfLevel, min, max))
        return VI_ERROR_PARAMETER2;
    if (rssmy_invalidViBooleanRange (rfLevelUnits))
        return VI_ERROR_PARAMETER3;
	if (rssmy_invalidViBooleanRange (rfLevelType))
        return VI_ERROR_PARAMETER4;

    sprintf (buffer, "%s %9.8f%s", levelComm[rfLevelType], rfLevel, levelUnits[rfLevelUnits]);
	CHECKERR( viWrite( instrSession, (ViByte *)buffer, strlen( buffer), VI_NULL));
	CHECKERR( rssmy_check_status( instrSession));

    return rssmy_status;
}

/*=========================================================================*/
/* Function: AF Setting                                                    */
/* Purpose:  This function sets the frequency of the AF generator.         */
/*=========================================================================*/
ViStatus _VI_FUNC rssmy_setAF (ViSession instrSession, ViReal64 afValue)
{
	ViStatus	rssmy_status = VI_SUCCESS;
	ViChar		buffer[BUFFER_SIZE];
	
	/*- Check input parameter ranges ----------------------------------------*/
    if (rssmy_invalidViReal64Range (afValue, 0.1, 5.0e5))
        return VI_ERROR_PARAMETER2;

    sprintf (buffer, "AF %lfHZ", afValue);
	CHECKERR( viWrite( instrSession, (ViByte *)buffer, strlen( buffer), VI_NULL));
    CHECKERR( rssmy_check_status( instrSession));

    return rssmy_status;
}

/*=========================================================================*/
/* Function: Special Settings                                              */
/* Purpose:  This function sets the reference frequency and the            */
/*           non-interrupting level setting                                */
/*=========================================================================*/
ViStatus _VI_FUNC rssmy_specialSetting (ViSession instrSession,
                                        ViBoolean refFreq, ViBoolean nonInter)
{
    ViStatus	rssmy_status = VI_SUCCESS;
	ViChar		buffer[BUFFER_SIZE];
	
	/*- Check input parameter ranges ----------------------------------------*/
	if (rssmy_invalidViBooleanRange (refFreq))
        return VI_ERROR_PARAMETER2;
	if (rssmy_invalidViBooleanRange (nonInter))
        return VI_ERROR_PARAMETER3;

    sprintf (buffer, "REF:%s;AT:%s", refFreqComm[refFreq], nonIntrComm[nonInter]);

    CHECKERR( viWrite( instrSession, (ViByte *)buffer, strlen( buffer), VI_NULL));
    CHECKERR( rssmy_check_status( instrSession));

    return rssmy_status;
}

/*=========================================================================*/
/* Function: AM Setting                                                    */
/* Purpose:  This function sets the amplitude modulation parameters.       */
/*=========================================================================*/
ViStatus _VI_FUNC rssmy_setAMMod (ViSession instrSession,
                                  ViInt32 amType, ViReal64 amDepth)
{
    ViStatus	rssmy_status = VI_SUCCESS;
	ViChar		buffer[BUFFER_SIZE];
	ViInt32		tmpCnt;

	/*- Check input parameter ranges ----------------------------------------*/
	if (rssmy_invalidViInt32Range (amType, 0, 5))
        return VI_ERROR_PARAMETER2;
	if (rssmy_invalidViReal64Range (amDepth, 0.0, 100.0))
        return VI_ERROR_PARAMETER3;

    tmpCnt = sprintf (buffer, "AM:%s", AMTypeComm[amType]);

    if (amType > 0)
        sprintf (buffer + tmpCnt, " %lfPCT", amDepth);

	CHECKERR( viWrite( instrSession, (ViByte *)buffer, strlen( buffer), VI_NULL));
    CHECKERR( rssmy_check_status( instrSession));

    return rssmy_status;
}

/*=========================================================================*/
/* Function: FM Setting                                                    */
/* Purpose:  This function sets the frequency modulation parameters.       */
/*=========================================================================*/
ViStatus _VI_FUNC rssmy_setFMMod (ViSession instrSession,
                                  ViInt32 frqType, ViReal64 fmDeviation)
{
    ViStatus	rssmy_status = VI_SUCCESS;
	ViChar		buffer[BUFFER_SIZE];
	ViInt32		tmpCnt;

	/*- Check input parameter ranges ----------------------------------------*/
	if (rssmy_invalidViInt32Range (frqType, 0, 5))
        return VI_ERROR_PARAMETER2;
	if (rssmy_invalidViReal64Range (fmDeviation, 0.0, 2.0e7))
        return VI_ERROR_PARAMETER3;

    tmpCnt = sprintf (buffer, "FM:%s", FMTypeComm[frqType]);
    
    if (frqType > 0)
        sprintf (buffer + tmpCnt, " %lfHZ", fmDeviation);

	CHECKERR( viWrite( instrSession, (ViByte *)buffer, strlen( buffer), VI_NULL));
    CHECKERR( rssmy_check_status( instrSession));
    
    return rssmy_status;
}

/*=========================================================================*/
/* Function: PM Setting                                                    */
/* Purpose:  This function sets the pulse modulation parameters.           */
/*=========================================================================*/
ViStatus _VI_FUNC rssmy_setPMMod (ViSession instrSession, ViInt32 pulseType)
{
	ViStatus	rssmy_status = VI_SUCCESS;
	ViChar		buffer[BUFFER_SIZE];

	/*- Check input parameter ranges ----------------------------------------*/
	if (rssmy_invalidViInt32Range (pulseType, 0, 2))
        return VI_ERROR_PARAMETER2;

    sprintf (buffer, "B:%s", PMTypeComm[pulseType]);

	CHECKERR( viWrite( instrSession, (ViByte *)buffer, strlen( buffer), VI_NULL));
    CHECKERR( rssmy_check_status( instrSession));

    return rssmy_status;
}

/*=========================================================================*/
/* Function: phM Setting                                                   */
/* Purpose:  This function sets the phase modulation parameters.           */
/*=========================================================================*/
ViStatus _VI_FUNC rssmy_setPhMMod (ViSession instrSession,
                                   ViInt32 phMType, ViReal64 phMDeviation)
{
	ViStatus	rssmy_status = VI_SUCCESS;
	ViChar		buffer[BUFFER_SIZE];
	ViInt32		tmpCnt;

	/*- Check input parameter ranges ----------------------------------------*/
	if (rssmy_invalidViInt32Range (phMType, 0, 3))
        return VI_ERROR_PARAMETER2;
    if (rssmy_invalidViReal64Range (phMDeviation, 0.0, 400.0))
        return VI_ERROR_PARAMETER3;
        
    tmpCnt = sprintf (buffer, "PHM:%s", phMTypeComm[phMType]);
    
    if (phMType > 0)
        sprintf (buffer + tmpCnt, " %lfRAD", phMDeviation);

	CHECKERR( viWrite( instrSession, (ViByte *)buffer, strlen( buffer), VI_NULL));
    CHECKERR( rssmy_check_status( instrSession));

    return rssmy_status;
}

/*=========================================================================*/
/* Function: Freq./Level Steps                                             */
/* Purpose:  This function sets the frequency and level variation steps    */
/*=========================================================================*/
ViStatus _VI_FUNC rssmy_setFreqLevelStep (ViSession instrSession,
                                          ViReal64 rfStep, ViReal64 afStep,
                                          ViReal64 levelStep)
{
	ViStatus	rssmy_status = VI_SUCCESS;
	ViChar		buffer[BUFFER_SIZE];
	
	/*- Check input parameter ranges ----------------------------------------*/
	if (rssmy_invalidViReal64Range (rfStep, 1.0, 1.0e8))
        return VI_ERROR_PARAMETER2;
    if (rssmy_invalidViReal64Range (afStep, 0.1, 1.0e3))
        return VI_ERROR_PARAMETER3;
	if (rssmy_invalidViReal64Range (levelStep, 0.1, 125.0))
        return VI_ERROR_PARAMETER4;
    
    sprintf (buffer, "RF:V %lfHZ;AF:V %lfHZ;L:V %lfDB", rfStep, afStep, levelStep);

    CHECKERR( viWrite( instrSession, (ViByte *)buffer, strlen( buffer), VI_NULL));
    CHECKERR( rssmy_check_status( instrSession));

    return rssmy_status;
}

/*=========================================================================*/
/* Function: Modulation Steps                                              */
/* Purpose:  This function sets the AM depth and FM deviation variation    */
/*           steps                                                         */
/*=========================================================================*/
ViStatus _VI_FUNC rssmy_setModulationStep (ViSession instrSession,
                                           ViReal64 amStep, ViReal64 fmStep,
                                           ViReal64 phMStep)
{
	ViStatus	rssmy_status = VI_SUCCESS;
	ViChar		buffer[BUFFER_SIZE];
	
	/*- Check input parameter ranges ----------------------------------------*/
	if (rssmy_invalidViReal64Range (amStep, 0.1, 100.0))
        return VI_ERROR_PARAMETER2;
	if (rssmy_invalidViReal64Range (fmStep, 10.0, 1.99e7))
        return VI_ERROR_PARAMETER3;
	if (rssmy_invalidViReal64Range (phMStep, 0.001, 39.9))
        return VI_ERROR_PARAMETER4;

    sprintf (buffer,"AM:V %lfP;FM:V %lfHZ;PHM:V %lfRAD", amStep, fmStep, phMStep);

    CHECKERR( viWrite( instrSession, (ViByte *)buffer, strlen( buffer), VI_NULL));
    CHECKERR( rssmy_check_status( instrSession));

    return rssmy_status;
}

/*=========================================================================*/
/* Function: RF Output                                                     */
/* Purpose:  This function enables/disables the output signal.             */
/*=========================================================================*/
ViStatus _VI_FUNC rssmy_rfOut (ViSession instrSession, ViBoolean rfOutput)
{
	ViStatus	rssmy_status = VI_SUCCESS;
	ViChar		buffer[BUFFER_SIZE];
	
	/*- Check input parameter ranges ----------------------------------------*/
	if (rssmy_invalidViBooleanRange (rfOutput))
        return VI_ERROR_PARAMETER2;

    sprintf (buffer, "L:%s", stateComm[rfOutput]);
    
    CHECKERR( viWrite( instrSession, (ViByte *)buffer, strlen( buffer), VI_NULL));
    CHECKERR( rssmy_check_status( instrSession));

    return rssmy_status;
}

/*=========================================================================*/
/* Function: AF Output                                                     */
/* Purpose:  This function enables/disables the AF output signal.          */
/*=========================================================================*/
ViStatus _VI_FUNC rssmy_afOut (ViSession instrSession, ViBoolean afOutput)
{
	ViStatus	rssmy_status = VI_SUCCESS;
	ViChar		buffer[BUFFER_SIZE];
	
	/*- Check input parameter ranges ----------------------------------------*/
	if (rssmy_invalidViBooleanRange (afOutput))
        return VI_ERROR_PARAMETER2;

    sprintf (buffer, "AF:%s", stateComm[afOutput]);

    CHECKERR( viWrite( instrSession, (ViByte *)buffer, strlen( buffer), VI_NULL));
    CHECKERR( rssmy_check_status( instrSession));

    return rssmy_status;
}

/*=========================================================================*/
/* Function: Variation Step                                                */
/* Purpose:  This function performs the step variation of the selected     */
/*           parameter in the specified direction.                         */
/*=========================================================================*/
ViStatus _VI_FUNC rssmy_singleVarStep (ViSession instrSession,
                                       ViInt32 parameter, ViBoolean direction)
{
    ViStatus	rssmy_status = VI_SUCCESS;
	ViChar		buffer[BUFFER_SIZE];
	
	/*- Check input parameter ranges ----------------------------------------*/
	if (rssmy_invalidViInt32Range (parameter, 0, 5))
        return VI_ERROR_PARAMETER2;
	if (rssmy_invalidViBooleanRange (direction))
        return VI_ERROR_PARAMETER3;

    sprintf (buffer, "%s:%s", directionComm[direction], parameterComm[parameter]);

    CHECKERR( viWrite( instrSession, (ViByte *)buffer, strlen( buffer), VI_NULL));
    CHECKERR( rssmy_check_status( instrSession));

    return rssmy_status;
}

/*===========================================================================*/
/* Function: Write To Instrument                                             */
/* Purpose:  This function writes a command string to the instrument.        */
/*===========================================================================*/
ViStatus _VI_FUNC rssmy_writeInstrData (ViSession instrSession, ViString writeBuffer)
{
    ViStatus rssmy_status = VI_SUCCESS;
    
    CHECKERR( viPrintf (instrSession, "%s", writeBuffer));

    return rssmy_status;
}

/*===========================================================================*/
/* Function: Read Instrument Buffer                                          */
/* Purpose:  This function reads the output buffer of the instrument.        */
/*===========================================================================*/
ViStatus _VI_FUNC rssmy_readInstrData (ViSession instrSession, ViInt32 numberBytesToRead,
                    ViChar _VI_FAR readBuffer[], ViPInt32 numBytesRead)
{
    ViStatus	rssmy_status = VI_SUCCESS;
	ViUInt32	tmpValue;
	
	if (numBytesRead)
		*numBytesRead = 0L;

    CHECKERR( viRead (instrSession, (ViByte *)readBuffer, numberBytesToRead, &tmpValue));

	if (numBytesRead)
		*numBytesRead = (ViInt32)tmpValue;

    return rssmy_status;
}

/*=========================================================================*/
/* Function: Reset                                                         */
/* Purpose:  This function performs the instrument reset operation         */
/*=========================================================================*/
ViStatus _VI_FUNC rssmy_reset (ViSession instrSession)
{
    ViStatus	rssmy_status = VI_SUCCESS;
	
	CHECKERR( viWrite( instrSession, (ViByte *)"*RST;*HDR 0", 11, VI_NULL));
	CHECKERR( rssmy_defaultInstrSetup (instrSession));
    CHECKERR( rssmy_check_status( instrSession));

    return rssmy_status;
}

/*===========================================================================*/
/* Function: Self Test                                                       */
/* Purpose:  This function executes the instrument self-test and returns     */
/*           the result. 													 */
/*===========================================================================*/
ViStatus _VI_FUNC rssmy_self_test (ViSession instrSession,
                                   ViPInt16 testResult,
                                   ViChar _VI_FAR testMessage[])
{
    ViStatus	rssmy_status = VI_SUCCESS;
    
	/*- Check input parameter ranges ----------------------------------------*/
    if (!testResult)
    	return VI_ERROR_PARAMETER2;

    CHECKERR( viWrite( instrSession, (ViByte *)"*TST?", 5, VI_NULL));
    CHECKERR( viScanf( instrSession, "%hd", testResult));

    if (testMessage)
		testMessage[0] = '\0';

	CHECKERR( rssmy_check_status( instrSession));

    return rssmy_status;
}

/*=========================================================================*/
/* Function: Diagnostics                                                   */
/* Purpose:  This function performs the internal voltage test.             */
/*=========================================================================*/
ViStatus _VI_FUNC rssmy_diagnostics (ViSession instrSession,
                                     ViInt32 testpoint, ViBoolean testState,
                                     ViPReal64 reading)
{
	ViStatus	rssmy_status = VI_SUCCESS;
	ViChar		buffer[BUFFER_SIZE];

    /*- Check input parameter ranges ----------------------------------------*/
	if (rssmy_invalidViInt32Range (testpoint, 1, 17))
        return VI_ERROR_PARAMETER2;
	if (rssmy_invalidViBooleanRange (testState))
        return VI_ERROR_PARAMETER3;
    
    if (testState)
    	{
    	if (!reading)
        	return VI_ERROR_PARAMETER4;
        
        sprintf (buffer, "TE:P %ld;TE:V?", testpoint);
        CHECKERR( viWrite( instrSession, (ViByte *)buffer, strlen( buffer), VI_NULL));
        CHECKERR( viRead( instrSession, (ViByte *)buffer, BUFFER_SIZE, VI_NULL));
        
        if (sscanf (buffer, "%lf", reading) != 1)
            return VI_ERROR_INSTR_INTERPRETING_RESPONSE;
    	}
    else
    	{
    	CHECKERR( viWrite( instrSession, (ViByte *)"TE:OFF", 6, VI_NULL));
    	}

	CHECKERR( rssmy_check_status( instrSession));

    return rssmy_status;
}

/*===========================================================================*/
/* Function: Revision Query                                                  */
/* Purpose:  This function returns the driver and instrument revisions.      */
/*           If the revision query function is not supported by the          */ 
/*           instrument, this function returns the warning                   */
/*           VI_WARN_NSUP_REV_QUERY.                                         */
/*===========================================================================*/
ViStatus _VI_FUNC rssmy_revision_query (ViSession instrSession,
                                        ViChar _VI_FAR driverRev[],
                                        ViChar _VI_FAR instrRev[])
{
    ViStatus rssmy_status = VI_SUCCESS;

    if (instrRev)
    	{
    	CHECKERR( viWrite( instrSession, (ViByte *)"*IDN?", 5, VI_NULL));
		CHECKERR( viScanf( instrSession, "%*[^,],%*[^,],%*[^,],%[^\n]", instrRev));
		}

    if (driverRev)
    	{
    	strcpy (driverRev, RSSMY_REVISION);
    	}

    return rssmy_status;
}

/*=========================================================================*/
/* Function: Error Query                                                   */
/* Purpose:  This function queries the instrument error queue.             */
/*=========================================================================*/
ViStatus _VI_FUNC rssmy_error_query (ViSession instrSession, ViPInt32 errorCode,
                                     ViChar _VI_FAR errorMessage[])
{
	ViStatus	rssmy_status = VI_SUCCESS;
	ViChar		buffer[BUFFER_SIZE];
	ViInt32		tmpValue;

	CHECKERR( viWrite (instrSession, (ViByte *)"E?", 2, VI_NULL));
	CHECKERR( viRead (instrSession, (ViByte *)buffer, BUFFER_SIZE, VI_NULL));
	sscanf(buffer, "%ld", &tmpValue);
	
	if (errorCode)
		*errorCode = (!tmpValue) ? 0 : -1;
		
	if (errorMessage)
		strncpy ( errorMessage, buffer, 256);
	
	CHECKERR( rssmy_check_status( instrSession));

    return rssmy_status;
}

/*=========================================================================*/
/* Function: Error Message                                                 */
/* Purpose:  This function returns a text message for a corresponding      */
/*           instrument driver error code.                                 */
/*=========================================================================*/
ViStatus _VI_FUNC rssmy_error_message (ViSession instrSession,
                                       ViStatus statusCode,
                                       ViChar _VI_FAR errMessage[])
{
    ViStatus rssmy_status = VI_SUCCESS;
    ViInt16 i;

    static rssmy_tStringValPair statusDescArray[] = {
        {VI_ERROR_PARAMETER1,   "ERROR: Parameter 1 out of range"},
        {VI_ERROR_PARAMETER2,   "ERROR: Parameter 2 out of range"},
        {VI_ERROR_PARAMETER3,   "ERROR: Parameter 3 out of range"},
        {VI_ERROR_PARAMETER4,   "ERROR: Parameter 4 out of range"},
        {VI_ERROR_FAIL_ID_QUERY,"ERROR: Identification query failed"},
        {VI_ERROR_INV_RESPONSE, "ERROR: Interpreting instrument response"},
        {VI_ERROR_INSTR_INTERPRETING_RESPONSE, "ERROR: Interpreting the instrument's response"},
        {RSSMY_ERROR_INSTR_SPECIFIC, "ERROR: Instrument specific error."},
        {VI_NULL, VI_NULL}
    };

    rssmy_status = viStatusDesc (instrSession, statusCode, errMessage);
    if (rssmy_status == VI_WARN_UNKNOWN_STATUS)
    	{
        for (i = 0; statusDescArray[i].stringName; i++)
        	{
            if (statusDescArray[i].stringVal == statusCode)
            	{
                strcpy (errMessage, statusDescArray[i].stringName);
                return (VI_SUCCESS);
            	}
        	}
        sprintf (errMessage, "Unknown Error 0x%08X", statusCode);
        return (VI_WARN_UNKNOWN_STATUS);
    	}
    
    rssmy_status = VI_SUCCESS;

    return rssmy_status;
}

/*=========================================================================*/
/* Function: Check For Status                                              */
/* Purpose:  This function specifies whether to check for an error after   */
/*           the each function call.                                       */
/*=========================================================================*/
ViStatus _VI_FUNC rssmy_checkStatus (ViSession instrSession,
                                     ViBoolean checkForStatus)
{
	ViStatus	rssmy_status = VI_SUCCESS;
	rssmy_instrRange instrPtr = VI_NULL;

    if (rssmy_invalidViBooleanRange (checkForStatus))
        return VI_ERROR_PARAMETER2;

    CHECKERR( viGetAttribute (instrSession, VI_ATTR_USER_DATA, &instrPtr));

    if (instrPtr)
    	instrPtr -> checkStatus = checkForStatus;

    return rssmy_status;
}

/*=========================================================================*/
/* Function: Close                                                         */
/* Purpose:  This function closes the instrument.                          */
/*=========================================================================*/
ViStatus _VI_FUNC rssmy_close (ViSession instrSession)
{
    ViStatus rssmy_status = VI_SUCCESS;
    rssmy_instrRange instrPtr;
    ViSession rmSession;

    CHECKERR( viGetAttribute (instrSession, VI_ATTR_RM_SESSION, &rmSession));
    CHECKERR( viGetAttribute (instrSession, VI_ATTR_USER_DATA, &instrPtr));

    if (instrPtr != NULL)
        free (instrPtr);
    
    rssmy_status = viClose (instrSession);
    viClose (rmSession);

    return rssmy_status;
}

/*****************************************************************************/
/*= UTILITY ROUTINES (Non-Exportable Functions) =============================*/
/*****************************************************************************/

/*===========================================================================*/
/* Function: Boolean Value Out Of Range - ViBoolean                          */
/* Purpose:  This function checks a Boolean to see if it is equal to VI_TRUE */
/*           or VI_FALSE. If the value is out of range, the return value is  */
/*           VI_TRUE, otherwise the return value is VI_FALSE.                */
/*===========================================================================*/
ViBoolean rssmy_invalidViBooleanRange (ViBoolean val)
{
    return (ViBoolean)((val != VI_FALSE && val != VI_TRUE) ? VI_TRUE : VI_FALSE);
}

/*===========================================================================*/
/* Function: Real (Double) Value Out Of Range - ViReal64                     */
/* Purpose:  This function checks a real (double) value to see if it lies    */  
/*           between a minimum and maximum value.  If the value is out of    */
/*           range, the return value is VI_TRUE, otherwise the return value  */
/*           is VI_FALSE.                                                    */
/*===========================================================================*/
ViBoolean rssmy_invalidViReal64Range (ViReal64 val, ViReal64 min, ViReal64 max)
{
    return (ViBoolean)((val < min || val > max) ? VI_TRUE : VI_FALSE);
}

/*===========================================================================*/
/* Function: Long Signed Integer Value Out Of Range - ViInt32                */
/* Purpose:  This function checks a long signed integer value to see if it   */  
/*           lies between a minimum and maximum value.  If the value is out  */
/*           of range, the return value is VI_TRUE, otherwise the return     */
/*           value is VI_FALSE.                                              */
/*===========================================================================*/
ViBoolean rssmy_invalidViInt32Range (ViInt32 val, ViInt32 min, ViInt32 max)
{
    return (ViBoolean)((val < min || val > max) ? VI_TRUE : VI_FALSE);
}

/*===========================================================================*/
/* Function: Initialize Clean Up                                             */
/* Purpose:  This function is used only by the rssmy_init function.  When    */
/*           an error is detected this function is called to close the       */
/*           open resource manager and instrument object sessions and to     */
/*           set the instrSession that is returned from rssmy_init to        */
/*           VI_NULL.                                                        */
/*===========================================================================*/
ViStatus rssmy_initCleanUp (ViSession openRMSession,
                    ViPSession openInstrSession, ViStatus currentStatus)
{
    rssmy_instrRange instrPtr;
    
    if (viGetAttribute (*openInstrSession, VI_ATTR_USER_DATA, &instrPtr) >= 0)
        if (instrPtr != NULL) 
            free (instrPtr);

    viClose (*openInstrSession);
    viClose (openRMSession);
    *openInstrSession = VI_NULL;

    return currentStatus;
}

/*===========================================================================*/
/* Function: Default Instrument Setup                                        */
/* Purpose:  This function sends a default setup to the instrument.  This    */
/*           function is called by the ifr68xx_reset operation and by the    */
/*           ifr68xx_init function if the reset option has not been          */
/*           selected.  This function is useful for configuring any          */
/*           instrument settings that are required by the rest of the        */
/*           instrument driver functions such as turning headers ON or OFF   */
/*           or using the long or short form for commands, queries, and data.*/                                    
/*===========================================================================*/
ViStatus rssmy_defaultInstrSetup (ViSession instrSession)
{
    ViStatus	rssmy_status = VI_SUCCESS;
    rssmy_instrRange instrPtr;
    
    /* Determine if the structure has been initialized for the current VISA  */
    /* Session and malloc if it has not.                                     */
    CHECKERR( viGetAttribute (instrSession, VI_ATTR_USER_DATA, &instrPtr));

    if (instrPtr == NULL) 
    	{
        instrPtr = malloc (sizeof (struct rssmy_statusDataRanges));
        }

    instrPtr -> checkStatus = VI_TRUE;
    strcpy (instrPtr -> instrDriverRevision, RSSMY_REVISION);

    CHECKERR( viSetAttribute (instrSession, VI_ATTR_USER_DATA, (ViUInt32)instrPtr));
    CHECKERR( viWrite (instrSession, (ViByte *)"HEA:OFF", 7, VI_NULL));

    return rssmy_status;
}

/*=========================================================================*/
/* Function: Status                                                        */
/* Purpose:  This function waits for completition of all running           */
/*           operations and checks for errors.                             */
/*=========================================================================*/
ViStatus rssmy_check_status (ViSession instrSession)
{
    ViStatus	rssmy_status;
    ViInt32		esr = 0;
    rssmy_instrRange instrPtr = VI_NULL;

	CHECKERR( viGetAttribute (instrSession, VI_ATTR_USER_DATA, &instrPtr));

	if (instrPtr -> checkStatus)
		{
		CHECKERR( viWrite (instrSession, (ViByte *)"*HDR 0;*ESR?", 12, VI_NULL));
		CHECKERR( viScanf( instrSession, "%hd", &esr));

		if (esr & IEEE_488_2_ERROR_BITS)
			return RSSMY_ERROR_INSTR_SPECIFIC;
		}

    return rssmy_status;
}

/*****************************************************************************/
/*=== END INSTRUMENT DRIVER SOURCE CODE =====================================*/
/*****************************************************************************/
